def module_from_str(s):
    if not s:
        raise ValueError('Empty module name')
    from_name = s.split('.')[-1]
    try:
        mod = __import__(s, fromlist=str(from_name))
        return mod
    except:
        import traceback
        tb = traceback.format_exc()
        raise ImportError("Unable to import %s with exception : \n[\n%s]" % (s, tb))

def class_from_str(s):
    if not s:
        raise ValueError('Empty class name')
    mod_name = '.'.join(s.split('.')[0:-1])
    class_name = s.split('.')[-1]
    try:
        mod = module_from_str(mod_name)
        return getattr(mod, class_name)
    except ImportError:
        raise
    except AttributeError as e:
        raise ImportError("Unable to import %s from module '%s': %s " % (class_name, mod_name, e.args))



from __future__ import absolute_import

EXTENSION = '.yaml'

class SerializationSafetyError(Exception):
    pass

def dump(obj, f, *args, **kwargs):
    from baiji.serialization.util.openlib import ensure_file_open_and_call
    return ensure_file_open_and_call(f, _dump, 'w', obj, *args, **kwargs)


def load(f, *args, **kwargs):
    from baiji.serialization.util.openlib import ensure_file_open_and_call
    return ensure_file_open_and_call(f, _load, 'r', *args, **kwargs)


def loads(s, *args, **kwargs):
    return _load(s, *args, **kwargs)


def dumps(obj, *args, **kwargs):
    import yaml
    import warnings
    safe_mode = kwargs.pop('safe', None)
    try:
        return yaml.safe_dump(obj, *args, **kwargs)
    except yaml.representer.RepresenterError as e:
        if safe_mode:
            raise SerializationSafetyError(*e.args)
        if safe_mode is None:
            warnings.warn(
                ('Unsafe YAML serialization. This will generate an error in the '
                 'future. Call with `safe=False` if you really want unsafe serialization.'),
                DeprecationWarning, stacklevel=2
            )
        # if safe_mode is None (not given) or False (explicitly set), fall back to unsafe behavior
        return yaml.dump(obj, *args, **kwargs)



def _dump(f, obj, *args, **kwargs):
    import yaml
    import warnings
    safe_mode = kwargs.pop('safe', None)
    try:
        return yaml.safe_dump(obj, f, *args, **kwargs)
    except yaml.representer.RepresenterError as e:
        if safe_mode:
            raise SerializationSafetyError(*e.args)
        if safe_mode is None:
            warnings.warn(
                ('Unsafe YAML serialization. This will generate an error in the '
                 'future. Call with `safe=False` if you really want unsafe serialization.'),
                DeprecationWarning, stacklevel=2
            )
        # if safe_mode is None (not given) or False (explicitly set), fall back to unsafe behavior
        return yaml.dump(obj, f, *args, **kwargs)


def _load(f, *args, **kwargs):
    import yaml
    import warnings
    safe_mode = kwargs.pop('safe', None)
    try:
        return yaml.safe_load(f, *args, **kwargs)
    except yaml.constructor.ConstructorError as e:
        if safe_mode:
            raise SerializationSafetyError(*e.args)
        if safe_mode is None:
            warnings.warn(
                ('Unsafe YAML serialization. This will generate an error in the '
                 'future. Call with `safe=False` if you really want unsafe serialization.'),
                DeprecationWarning, stacklevel=2
            )
        # if safe_mode is None (not given) or False (explicitly set), fall back to unsafe behavior
        return yaml.load(f, *args, **kwargs)



import unittest
from baiji.serialization import yaml

class UnsafeToDump(object):
    """Python classes should not by default be YAML dumpable in safe mode"""
    pass

unsafe_to_load = "!!python/name:baiji.serialization.test_yaml.UnsafeToDump ''\n"

class TestYAML(unittest.TestCase):

    def test_yaml_unsafe_dumps(self):
        self.assertEqual(yaml.dumps(UnsafeToDump), unsafe_to_load)

    def test_yaml_unsafe_loads_uses_load(self):
        self.assertEqual(yaml.loads(unsafe_to_load), UnsafeToDump)

    def test_yaml_safe_dump_raises_on_unsafe(self):
        self.assertRaises(
            yaml.SerializationSafetyError,
            yaml.dumps, UnsafeToDump, safe=True)

    def test_yaml_safe_load_raises_on_unsafe(self):
        self.assertRaises(
            yaml.SerializationSafetyError,
            yaml.loads, unsafe_to_load, safe=True)



from cPickle import UnpicklingError # We import this to expose it to our importers pylint: disable=unused-import

EXTENSION = '.pkl'

def dump(obj, f):
    from baiji.serialization.util.openlib import ensure_file_open_and_call
    return ensure_file_open_and_call(f, _dump, 'wb', obj)

def load(f, *args, **kwargs):
    from baiji.serialization.util.openlib import ensure_file_open_and_call
    return ensure_file_open_and_call(f, _load, 'rb', *args, **kwargs)

def loads(s, *args, **kwargs):
    import cPickle as pickle
    return pickle.loads(s, *args, **kwargs)

def dumps(obj):
    import cPickle as pickle
    return pickle.dumps(obj, pickle.HIGHEST_PROTOCOL)

def _dump(f, obj):
    import cPickle as pickle
    return pickle.dump(obj, f, pickle.HIGHEST_PROTOCOL)

def _load(f, constructors=None):
    '''
    constructors: A dictionary mapping strings to callables, which are
      consulted as additional constructors during unpickling. This is useful
      for interposing an alternative constructor, or for supporting class
      names which have been changed. This is implemented via the cPickle
      `Unpickler`'s `find_global` attribute, which is documented here,
      albeit quite densely:
      https://docs.python.org/2/library/pickle.html#subclassing-unpicklers
    '''
    import cPickle as pickle
    from baiji.serialization.util.importlib import class_from_str

    def unpickler_find_global(module_name, class_name):
        fully_qualified_name = '{}.{}'.format(module_name, class_name)
        try:
            return constructors[fully_qualified_name]
        except KeyError:
            # It would be tempting to delegate to the original `find_global`.
            # Unfortunately, `Unpickler` does not expose the default
            # implementation. Thanks, Obama.
            return class_from_str(fully_qualified_name)

    unpickler = pickle.Unpickler(f)
    if constructors:
        unpickler.find_global = unpickler_find_global
    return unpickler.load()



from __future__ import absolute_import
import simplejson as json

EXTENSION = '.json'
ENCODE_PRIMITIVES_BY_DEFAULT = False

def dump(obj, f, *args, **kwargs):
    from baiji.serialization.util.openlib import ensure_file_open_and_call
    return ensure_file_open_and_call(f, _dump, 'w', obj, *args, **kwargs)

def dumps(*args, **kwargs):
    return json.dumps(*args, **_dump_args(kwargs))

def _dump(f, obj, *args, **kwargs):
    return json.dump(obj, f, *args, **_dump_args(kwargs))

def load(f, *args, **kwargs):
    from baiji.serialization.util.openlib import ensure_file_open_and_call
    return ensure_file_open_and_call(f, _load, 'r', *args, **kwargs)

def _load(f, *args, **kwargs):
    return json.load(f, *args, **_load_args(kwargs))

def loads(*args, **kwargs):
    return json.loads(*args, **_load_args(kwargs))

class MethodListCaller(object):
    '''
    This is an internal class that lets the JSON(En,De)coder classes
    and their subclasses easily register a list of methods to try, which
    will then be called in order until one of them succeeds.
    '''
    def register(self, method, index=-1):
        if not hasattr(self, 'method_list'):
            self.clear()
        if index == -1:
            index = len(self.method_list)
        self.method_list.insert(index, method)

    def clear(self):
        # be defensive if someone forgets to call super pylint: disable=attribute-defined-outside-init
        self.method_list = []

    def __call__(self, x):
        '''
        Call the methods in method_list until one of them returns something other than None
        and return that as the result of the call.
        '''
        import six
        try:
            return next(six.moves.filter(lambda x: x is not None, (f(x) for f in self.method_list)))
        except StopIteration:
            return self.default(x)

    def default(self, x):
        '''
        If none of the methods returned something, then return this default
        '''
        return x

class JSONDecoder(MethodListCaller):
    '''
    Instances may be passed to simplejson as object_hook to decode json objects.
    The decoders will be called in order. The first one to return something other
    than None wins. The decoders will be given a dict to parse, something like:

        {
            "__ndarray__": [
                    [859.0, 859.0],
                    [217.0, 106.0],
                    [302.0, 140.0]
                ],
            "dtype": "float32",
            "shape": [3, 2]
        }

    Note that for object_hook, if we want to do nothing, we return the dict unchanged
    and the json is decoded as a plain old dict; this behavior is the default of
    MethodListCaller.
    '''
    def __init__(self):
        self.register(self.decode_numpy)
        self.register(self.decode_scipy)
        self.register(self.decode)

    def decode(self, dct):
        '''
        In a subclass, either override this or add some decode functions and
        override __init__ to register them
        '''
        pass

    def decode_numpy(self, dct):
        if '__ndarray__' in dct:
            try:
                import numpy as np
            except ImportError:
                raise ImportError("JSON file contains numpy arrays; install numpy to load it")
            if 'dtype' in dct:
                dtype = np.dtype(dct['dtype'])
            else:
                dtype = np.float64
            return np.array(dct['__ndarray__'], dtype=dtype)

    def decode_scipy(self, dct):
        if '__scipy.sparse.sparsematrix__' in dct:
            if not 'dtype' in dct and 'shape' in dct and 'data' in dct and 'format' in dct and 'row' in dct and 'col' in dct:
                return dct
            try:
                import numpy as np
                import scipy.sparse as sp
            except ImportError:
                raise ImportError("JSON file contains scipy.sparse arrays; install numpy and scipy to load it")
            coo = sp.coo_matrix((dct['data'], (dct['row'], dct['col'])), shape=dct['shape'], dtype=np.dtype(dct['dtype']))
            return coo.asformat(dct['format'])


def _load_args(kwargs):
    if 'object_hook' in kwargs:
        raise ValueError("Instead of explicitly setting object_hook, subclass JSONDecoder and pass it as decoder")
    if 'decoder' in kwargs:
        kwargs['object_hook'] = kwargs['decoder']
        del kwargs['decoder']
    else:
        kwargs['object_hook'] = JSONDecoder()
    return kwargs


class JSONEncoder(MethodListCaller):
    '''
    Instances may be passed to simplejson as default to encode json objects.
    The encoders will be called in order. The first one to return something other
    than None wins. The encoders will be given an object to encore and should return
    a dict, something like:

        {
            "__ndarray__": [
                    [859.0, 859.0],
                    [217.0, 106.0],
                    [302.0, 140.0]
                ],
            "dtype": "float32",
            "shape": [3, 2]
        }

    Note that for default, if we want to do nothing, we return None and the object
    is encoded as best as simplejson can (which is often by throwing a TypeError).
    We override MethodListCaller.default to get this behavior.
    '''
    def __init__(self, primitive=ENCODE_PRIMITIVES_BY_DEFAULT):
        self.primitive = primitive
        self.register(self.encode_numpy)
        self.register(self.encode_scipy)
        self.register(self.encode)

    def default(self, x):
        return None

    def encode(self, obj):
        '''
        In a subclass, either override this or add some encode functions and
        override __init__ to register them
        '''
        pass

    def encode_numpy(self, obj):
        try:
            import numpy as np
            if isinstance(obj, np.ndarray):
                if self.primitive:
                    return obj.tolist()
                else:
                    return {
                        '__ndarray__': obj.tolist(),
                        'dtype': obj.dtype.name,
                        'shape': obj.shape,
                    }
            elif isinstance(obj, (np.bool8, np.bool_)):
                return bool(obj)
            elif isinstance(obj, (np.half, np.single, np.double, np.float_, np.longfloat, np.float16, np.float32, np.float64)) or (hasattr(np, 'float128') and isinstance(obj, np.float128)):
                return float(obj)
            elif isinstance(obj, (np.byte, np.short, np.intc, np.int_, np.longlong, np.intp, np.int8, np.int16, np.int32, np.int64, np.ubyte, np.ushort, np.uintc, np.uint, np.ulonglong, np.uintp, np.uint8, np.uint16, np.uint32, np.uint64)):
                return int(obj)
            else:
                return None
        except ImportError:
            # Clearly there won't be any numpy arrays to encode...
            return None

    def encode_scipy(self, obj):
        try:
            import scipy.sparse as sp
            if sp.isspmatrix(obj):
                coo = obj.tocoo()
                return {
                    '__scipy.sparse.sparsematrix__': True,
                    'format': obj.getformat(),
                    'dtype': obj.dtype.name,
                    'shape': obj.shape,
                    'data': coo.data,
                    'row': coo.row,
                    'col': coo.col,
                }
            else:
                return None
        except ImportError:
            # Clearly there won't be any scipy.sparse arrays to encode...
            return None


def _dump_args(kwargs):
    if 'default' in kwargs:
        raise ValueError("Instead of explicitly setting default, subclass JSONEncoder and pass it as encoder")
    if 'encoder' in kwargs:
        kwargs['default'] = kwargs['encoder']
        del kwargs['encoder']
    else:
        kwargs['default'] = JSONEncoder(primitive=kwargs.get('primitive', ENCODE_PRIMITIVES_BY_DEFAULT))
    if 'primitive' in kwargs:
        del kwargs['primitive']
    kwargs['for_json'] = True
    return kwargs



import unittest

class TestCSV(unittest.TestCase):

    def test_collection_csv_serializer_renders_correct_rows(self):
        from baiji.serialization.csv import CSVCollectionSerializer

        example_data = {
            'one': {'foo': 'baz1', 'bar': 'inga1'},
            'three': {'foo': 'baz3', 'bar': 'inga3'},
            'two': {'foo': 'baz2', 'bar': 'inga2'},
        }
        ordering = ['one', 'two', 'three']

        serializer = CSVCollectionSerializer(
            collection=example_data,
            row_ordering=ordering
        )

        expected = [
            ['', 'bar', 'foo'],
            ['one', 'inga1', 'baz1'],
            ['two', 'inga2', 'baz2'],
            ['three', 'inga3', 'baz3'],
        ]

        self.assertEqual(serializer.render(), expected)

    def test_collection_csv_serializer_works_with_row_ordering_np_array(self):
        import numpy as np
        from baiji.serialization.csv import CSVCollectionSerializer

        example_data = {
            'one': {'foo': 'baz1', 'bar': 'inga1'},
            'three': {'foo': 'baz3', 'bar': 'inga3'},
            'two': {'foo': 'baz2', 'bar': 'inga2'},
        }
        ordering = np.array(['one', 'two', 'three'])

        serializer = CSVCollectionSerializer(
            collection=example_data,
            row_ordering=ordering
        )

        expected = [
            ['', 'bar', 'foo'],
            ['one', 'inga1', 'baz1'],
            ['two', 'inga2', 'baz2'],
            ['three', 'inga3', 'baz3'],
        ]

        self.assertEqual(serializer.render(), expected)



import io
try:
    # Python 2: "file" is built-in
    file_types = file, io.IOBase
except NameError:
    # Python 3: "file" fully replased with IOBase
    file_types = (io.IOBase,)

def ensure_file_open_and_call(path_or_fp, fn, mode='r', *args, **kwargs):
    import six
    from baiji import s3

    if isinstance(path_or_fp, six.string_types):
        with s3.open(path_or_fp, mode) as f:
            return fn(f, *args, **kwargs)
    elif isinstance(path_or_fp, file_types) or (hasattr(path_or_fp, 'read') and hasattr(path_or_fp, 'seek')):
        result = fn(path_or_fp, *args, **kwargs)
        if hasattr(path_or_fp, 'flush'):
            path_or_fp.flush()
        return result
    else:
        raise ValueError('Object {} does not appear to be a path or a file like object'.format(path_or_fp))



from __future__ import absolute_import
import six

__all__ = ['load', 'dump', 'dumps', 'EXTENSION']

EXTENSION = '.csv'


def load(f, *args, **kwargs):
    from baiji.serialization.util.openlib import ensure_file_open_and_call
    return ensure_file_open_and_call(f, _load, mode='rb', *args, **kwargs)


def dump(obj, f):
    from baiji.serialization.util.openlib import ensure_file_open_and_call
    return ensure_file_open_and_call(f, _dump, mode='wb', obj=obj)

def dumps(obj):
    import StringIO
    output = StringIO.StringIO()
    _dump(output, obj)
    out_string = output.getvalue()
    output.close()
    return out_string

def _load(f, header_row=True, header_row_transformer=lambda x: x):
    '''
    header_row_transformer: Give the caller a chance to rewrite the header row.
      Accepts one argument, a sequence of field names, and should return a
      modified sequence.

    '''
    import csv

    reader = csv.reader(f)
    line_number = 1
    result = []

    if header_row:
        field_names = next(reader)
        field_names = header_row_transformer(field_names)
        line_number += 1

        for row_values in reader:
            if len(row_values) != len(field_names):
                raise ValueError("Header row contains %s items but line %s contains %s" % \
                    (len(field_names), line_number, len(row_values)))

            result.append({k: v for k, v in zip(field_names, row_values)})
            line_number += 1

    else:
        for row_values in reader:
            result.append({i: v for i, v in enumerate(row_values)})
            line_number += 1

    return result


def _dump(f, obj):
    '''
    Per the docs for csv module, use this with binary mode on platforms
    where it matters:

        from baiji.serialization import csv
        with open(file, 'wb') as f:
            csv.dump(obj, f)

        or

        csv.dump(file, obj)

    '''
    import csv
    if not isinstance(obj, list):
        raise ValueError('obj should be a list of lists or tuples')
    if not all([isinstance(x, tuple) or isinstance(x, list) for x in obj]):
        raise ValueError('obj should be a list of lists or tuples')
    writer = csv.writer(f)
    for item in obj:
        writer.writerow(item)


class CSVSerializer(object):
    '''
    Simple CSV serializer. Subclasses can support serializing arrays of
    arbitrary objects, using their own serialization format. Subclasses
    may also set header to an array of tuples which will be used as the
    header content.

    '''
    header = []
    def __init__(self, data):
        self._data = data
    @property
    def body(self):
        if hasattr(self, 'format'):
            return self.format(self._data)
        else:
            return self._data
    def render(self):
        return self.header + self.body
    def dump(self, f):
        # Delegate to baiji.serialization.csv.dump()
        dump(self.render(), f)


class CSVCollectionSerializer(CSVSerializer):
    '''
    Serialize to CSV from a collection of dicts. Dicts should have the
    same keys, which become the column headings.

    '''
    def __init__(self, collection, row_ordering=None):
        '''
        row_ordering: When collection is a dictionary, an optional array
          of keys specifying the order in which to emit the items.

        '''
        super(CSVCollectionSerializer, self).__init__(collection)
        self.keys = self.compute_keys(collection)
        if isinstance(collection, dict):
            self.header = [[''] + self.keys]
        else:
            self.header = [self.keys]
        self.row_ordering = row_ordering

    @classmethod
    def compute_keys(cls, collection):
        if isinstance(collection, dict):
            first_value = next(six.itervalues(collection))
        else:
            first_value = collection[0]
        result = sorted(first_value.keys())

        # Make sure the keys are consistent.
        if isinstance(collection, dict):
            keyed_collection = collection
        else:
            keyed_collection = {index: value for index, value in enumerate(collection)}

        expected = set(result)
        for key, item in six.iteritems(keyed_collection):
            if set(item.keys()) != expected:
                message = 'Item %s had different keys (got %s, expected %s)' % \
                    (key, ' '.join(item.keys()), ' '.join(expected))
                raise ValueError(message)

        return result

    def format(self, collection):
        if isinstance(collection, dict):
            if self.row_ordering is not None:
                row_ordering = self.row_ordering
            else:
                row_ordering = collection.keys()
            row_heads = [[key] for key in row_ordering]
            rows = [collection[key] for key in row_ordering]
        else:
            row_heads = [[] for _ in range(len(collection))]
            rows = collection

        return [
            row_head +
            [
                row[k]
                for k in self.keys
            ]
            for row_head, row in zip(row_heads, rows)
        ]



from __future__ import print_function
# https://coderwall.com/p/qawuyq
# Thanks James.

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError):
    long_description = ''
    print('warning: pandoc or pypandoc does not seem to be installed; using empty long_description')

import os
requirements_file = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'requirements.txt')
with open(requirements_file, 'r') as f:
    install_requires = [x.strip() for x in f.readlines()]

import importlib
from setuptools import setup

setup(
    name='metabaiji-serialization',
    version=importlib.import_module('baiji.serialization').__version__,
    author='Body Labs',
    author_email='github@paulmelnikow.com',
    description='Read and write common file formats to Amazon S3 and local files',
    long_description=long_description,
    url='https://github.com/metabolize/baiji-serialization',
    license='Apache',
    packages=[
        'baiji',
        'baiji/serialization',
        'baiji/serialization/util',
    ],
    install_requires=install_requires,
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
    ]
)



import unittest
import os
from baiji.serialization import json

class TestJson(unittest.TestCase):

    def setUp(self):
        import tempfile
        self.tmp_dir = tempfile.mkdtemp('baiji-serialization-json')

    def tearDown(self):
        import shutil
        shutil.rmtree(self.tmp_dir, ignore_errors=True)

    def test_json_dumps(self):
        '''Examples from json docs'''
        self.assertEqual(
            json.dumps(['foo', {'bar': ('baz', None, 1.0, 2)}]),
            r'["foo", {"bar": ["baz", null, 1.0, 2]}]')
        self.assertEqual(json.dumps("\"foo\bar"), r'"\"foo\bar"')
        self.assertEqual(json.dumps(u'\u1234'), r'"\u1234"')
        self.assertEqual(json.dumps('\\'), r'"\\"')
        self.assertEqual(
            json.dumps({"c": 0, "b": 0, "a": 0}, sort_keys=True),
            r'{"a": 0, "b": 0, "c": 0}')
        self.assertEqual(
            json.dumps([1, 2, 3, {'4': 5, '6': 7}], separators=(',', ':')),
            r'[1,2,3,{"4":5,"6":7}]')
        self.assertEqual(
            json.dumps({'4': 5, '6': 7}, sort_keys=True, indent=4, separators=(',', ': ')),
            '{\n    "4": 5,\n    "6": 7\n}')

    def test_json_dump_stringio(self):
        from six import StringIO
        io = StringIO()
        json.dump(['streaming API'], io)
        self.assertEqual(io.getvalue(), r'["streaming API"]')

    def test_json_dump_file(self):
        path = os.path.join(self.tmp_dir, "test_json_dump_file.json")
        with open(path, 'w') as f:
            json.dump(['File Test'], f)
        with open(path, 'r') as f:
            self.assertEqual(f.read(), r'["File Test"]')

    def test_json_dump_path(self):
        path = os.path.join(self.tmp_dir, "test_json_dump_path.json")
        json.dump(['File Test'], path)
        with open(path, 'r') as f:
            self.assertEqual(f.read(), r'["File Test"]')

    def test_json_loads(self):
        self.assertEqual(
            json.loads('["foo", {"bar":["baz", null, 1.0, 2]}]'),
            [u'foo', {u'bar': [u'baz', None, 1.0, 2]}])
        self.assertEqual(
            json.loads('"\\"foo\\bar"'),
            u'"foo\x08ar')

    def test_json_load_stringio(self):
        from six import StringIO
        io = StringIO('["streaming API"]')
        self.assertEqual(json.load(io), [u'streaming API'])

    def test_json_load_file(self):
        path = os.path.join(self.tmp_dir, "test_json_load_file.json")
        with open(path, 'w') as f:
            f.write(r'["File Test"]')
        with open(path, 'r') as f:
            self.assertEqual(json.load(f), [u'File Test'])

    def test_json_load_path(self):
        path = os.path.join(self.tmp_dir, "test_json_load_path.json")
        with open(path, 'w') as f:
            f.write(r'["File Test"]')
        self.assertEqual(json.load(path), [u'File Test'])


    def test_json_load_ndarray_tricks_compatible_1d(self):
        import numpy as np
        res = json.loads('{"foo": {"__ndarray__": [859.033935546875, 859.033935546875], "dtype": "float32", "shape": [2]}}')
        res_array = res["foo"]
        self.assertIsInstance(res_array, np.ndarray)
        self.assertEqual(res_array.shape, (2, ))
        self.assertEqual(res_array.dtype, np.float32)
        np.testing.assert_equal(res_array, np.array([859.033935546875, 859.033935546875]))

    def test_json_load_ndarray_tricks_compatible_2d(self):
        import numpy as np
        res = json.loads('{"foo": {"__ndarray__": [[859.0, 859.0], [217.0, 106.0], [302.0, 140.0]], "dtype": "float32", "shape": [3, 2]}}')
        res_array = res["foo"]
        self.assertIsInstance(res_array, np.ndarray)
        self.assertEqual(res_array.shape, (3, 2))
        self.assertEqual(res_array.dtype, np.float32)
        np.testing.assert_almost_equal(res_array, np.array([[859.0, 859.0], [217.0, 106.0], [302.0, 140.0]]))

    def test_json_dump_ndarray_tricks_compatible(self):
        import six
        import numpy as np
        if six.PY2:
            expected = r'{"foo": {"dtype": "float32", "shape": [3, 2], "__ndarray__": [[859.0, 859.0], [217.0, 106.0], [302.0, 140.0]]}}'
        else:
            expected = r'{"foo": {"__ndarray__": [[859.0, 859.0], [217.0, 106.0], [302.0, 140.0]], "dtype": "float32", "shape": [3, 2]}}'
        self.assertEqual(
            json.dumps({"foo": np.array([[859.0, 859.0], [217.0, 106.0], [302.0, 140.0]], dtype=np.float32)}),
            expected)

    def test_json_dump_ndarray_tricks_compatible_primitive_option(self):
        import numpy as np
        self.assertEqual(
            json.dumps({"foo": np.array([[859.0, 859.0], [217.0, 106.0], [302.0, 140.0]], dtype=np.float32)}, primitive=True),
            r'{"foo": [[859.0, 859.0], [217.0, 106.0], [302.0, 140.0]]}')

    def test_json_load_sparse_matrix(self):
        import numpy as np
        import scipy.sparse as sp
        res = json.loads(r'{"foo": {"format": "dia", "dtype": "float32", "shape": [3, 3], "__scipy.sparse.sparsematrix__": true, "data": {"dtype": "float64", "shape": [3], "__ndarray__": [1.0, 1.0, 1.0]}, "col": {"dtype": "int32", "shape": [3], "__ndarray__": [0, 1, 2]}, "row": {"dtype": "int32", "shape": [3], "__ndarray__": [0, 1, 2]}}}')
        res_array = res["foo"]
        self.assertIsInstance(res_array, sp.dia.dia_matrix)
        self.assertEqual(res_array.shape, (3, 3))
        self.assertEqual(res_array.dtype, np.float32)
        np.testing.assert_almost_equal(res_array.todense(), np.eye(3))

    def test_json_dump_sparse_matrix(self):
        import six
        import scipy.sparse as sp
        if six.PY2:
            expected = r'{"foo": {"format": "dia", "dtype": "float64", "shape": [3, 3], "__scipy.sparse.sparsematrix__": true, "data": {"dtype": "float64", "shape": [3], "__ndarray__": [1.0, 1.0, 1.0]}, "col": {"dtype": "int32", "shape": [3], "__ndarray__": [0, 1, 2]}, "row": {"dtype": "int32", "shape": [3], "__ndarray__": [0, 1, 2]}}}'
        else:
            expected = r'{"foo": {"__scipy.sparse.sparsematrix__": true, "format": "dia", "dtype": "float64", "shape": [3, 3], "data": {"__ndarray__": [1.0, 1.0, 1.0], "dtype": "float64", "shape": [3]}, "row": {"__ndarray__": [0, 1, 2], "dtype": "int32", "shape": [3]}, "col": {"__ndarray__": [0, 1, 2], "dtype": "int32", "shape": [3]}}}'
        self.assertEqual(json.dumps({"foo": sp.eye(3)}), expected)



# Publish into the `baiji` namespace.
#
# http://stackoverflow.com/a/1676069/893113
# https://www.python.org/dev/peps/pep-0420/

from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)
